--define some constants used for welcome messages etc
local addonName = "ClassicBlacklist"
local addonVersion = "1.0.0";
local messageFont = "|caF45b5B0" 
local contributors = "Contributors: Joshpbarron - Hazedesun";
local defaultReason = "no reason"
local currentName = ""
local versionCheckString = "reportversion"
local syncString = "sync"

--create the main frame that is used for the addon
local MainFrame = CreateFrame("Frame")

--register some events for use later on
MainFrame:RegisterEvent("ADDON_LOADED");
MainFrame:RegisterEvent("GROUP_ROSTER_UPDATE")
MainFrame:RegisterEvent("IGNORELIST_UPDATE");
MainFrame:RegisterEvent("CHAT_MSG_ADDON");
MainFrame:SetScript("OnEvent", MainFrame.OnEvent)

--this handles the events we registered above.
MainFrame:SetScript("OnEvent", function(self, event, ...)
	local args = {...}
   if event == "ADDON_LOADED" then
      if args[1] == addonName then
         cblPrint( addonName .. " " .. addonVersion)
         cblPrint( contributors);
         if not(C_ChatInfo.RegisterAddonMessagePrefix(addonName)) then
            cblPrint("Failed to register addon prefix - some functionality restricted.")
         end
         if firstLoadDone == nil then
            cblPrint("First time load - setting up.");
            printHelp()
            firstLoadDone = true
         end
         if firstLoadDoneThisChar == nil then
            cblPrint("First time load for this char! - setting up.");
            firstLoadDoneThisChar = true
         end
         if (ignoredReasons == nil) then
            ignoredReasons = {}
         end
      end
   end
   --this handles the leave prompt
   if event == "GROUP_ROSTER_UPDATE" then
      local ignored = {}
      for i = 1, C_FriendList.GetNumIgnores() do
         local name = C_FriendList.GetIgnoreName(i)
         ignored[Ambiguate(name, "short")] = true
      end

      C_Timer.After(1, function()
         local names = GetHomePartyInfo()
         local name = ""
         if not(names == nil) then
            for i = 1, #names do
               if ignored[names[i]] then
                  if (not(names[i] == nil)) then
                     name = string.lower(Ambiguate(names[i], "short"))
                     reason = ignoredReasons[name]
                     StaticPopup_Show("CONFIRM_LEAVE_IGNORE", name, reason)
                  end
                  break
               end
            end
         end
      end)
   end
   --when somebody is added to the ignore list - ask the user to enter a reason
   if event == "IGNORELIST_UPDATE" then
      --first get a list of all the names who have been ignored.
      local ignoredPlayers = {}
      for i = 1, C_FriendList.GetNumIgnores() do
         local name = C_FriendList.GetIgnoreName(i)
         local nameFormatted = string.lower(Ambiguate(name, "short"))
         ignoredPlayers[nameFormatted] = true
         if (ignoredReasons[nameFormatted] == nil) then
            --here we can ask the user to add a reason
            ignoredReasons[nameFormatted] = defaultReason
            currentName = nameFormatted
            StaticPopup_Show("NEW_IGNORE_ADDED", nameFormatted)
         end
      end 
      --we can then sanitize the reasons if somebody is not on the ignore list any more
      for key,value in pairs(ignoredReasons) do
         local found = 0
         for i = 1, C_FriendList.GetNumIgnores() do
            local name = C_FriendList.GetIgnoreName(i)
            local nameFormatted = string.lower(Ambiguate(name, "short"))
            if (key == nameFormatted) then
               found = 1
            end
         end
         if found == 0 then
            cblPrint("Removing reason for character: " .. key)
            ignoredReasons[key] = nil
         end
      end
   end
   --triggered when a message is received from another addon
   if event == "CHAT_MSG_ADDON" then
      --arg1 prefix, arg2 message, arg3 type, arg4 sender
      --if the message was received from cbl.
      if (args[1] == addonName) then
         local words = {}
         for word in args[2]:gmatch("%w+") do 
            table.insert(words, word)
         end
         --if the message came in via guild
         if (args[3] == "GUILD") then
            if (words[1] == versionCheckString) then
               C_ChatInfo.SendAddonMessage( addonName, versionCheckString .. " ".. addonVersion, "WHISPER", args[4])
            end
            if (words[1] == syncString) then
               for key,value in pairs(ignoredReasons) do
                  C_ChatInfo.SendAddonMessage( addonName, syncString .. " " .. key .. ":" .. value, "WHISPER", args[4])
               end
               
            end
         end
         --if the addon came in via whisper
         if (args[3] == "WHISPER") then
            if (words[1] == versionCheckString) then
               cblPrint(args[4] .. " has version" .. string.gsub(args[2], versionCheckString, ""))
            end
            if (words[1] == syncString) then
               local sentData = string.gsub(args[2], syncString, "")
               local name, reason = sentData:match("([^,]+):([^,]+)")
               name = string.gsub(name, " ", "")
               --cblPrint(args[4] .. " sent:" .. name .. " - " .. reason)
               ignoreAndAddReason(name, reason)
            end
         end
      end
   end

end);

function cblPrint (message)
   print(messageFont .. message)
end

StaticPopupDialogs.CONFIRM_LEAVE_IGNORE = {
text = "%s is on your ignore list. \n Reason: %s \n Do you want to leave this group?",
button1 = YES,
button2 = NO,
OnAccept = LeaveParty,
whileDead = 1, 
hideOnEscape = 1, 
showAlert = 1,
}

StaticPopupDialogs.NEW_IGNORE_ADDED = {
text = "%s was added to your ignore list. \n Please enter a reason:",
hasEditBox = 1,
button1 = YES,
button2 = NO,
OnShow = function (self)
   self.editBox:SetText(defaultReason)
end,
OnAccept = function (self)
   local enteredReason = self.editBox:GetText()
   -- do whatever you want with it
   addReason(currentName, enteredReason)
   currentName = nil
end,
whileDead = 1, 
showAlert = 1,
}

function ignoreAndAddReason(name, reason)
   if (C_FriendList.AddIgnore(name)) then
      addReason(name, reason)
   end
end

function addReason (name, reason)
   if not(name == nil) and not(reason == nil) then
      ignoredReasons[name] = reason
      cblPrint(name .. " was added to ignore, reason '" .. reason .. "'")
   end
end

function printHelp ()
   cblPrint("ClassicBlacklist reads your ignore list and warns you if anyone from the list joins the same party/raid as you")
   cblPrint("/cbl show to show the current logged reasons.")
   cblPrint("/cbl purge to clear all the current logged reasons.")
   cblPrint("/cbl check to print the versions of all online guild members.")
   cblPrint("/cbl gsync to show the ignore list of all guild members.")
end

SLASH_CBL1 = "/cbl"
SlashCmdList["CBL"] = function(msg)
   msg = string.lower(msg)
   words = {}
   for word in msg:gmatch("%w+") do 
      table.insert(words, word)
   end
   if (words[1] == "help") then
      printHelp()
   elseif (words[1] == "show") then
      cblPrint(  "Showing (" .. #ignoredReasons ..") recorded reasons:")
      for key,value in pairs(ignoredReasons) do
         cblPrint(key .. " - reason: " .. value)
      end
   elseif (words[1] == "purge") then 
      for key in pairs (ignoredReasons) do
         ignoredReasons[key] = nil
      end
      cblPrint(  "All entries purged")
   elseif (words[1] == "check") then
      if not(C_ChatInfo.SendAddonMessage( addonName, versionCheckString, "GUILD" )) then
         cblPrint("Error - check failed!")
      end
   elseif (words[1] == "gsync") then
      if not(C_ChatInfo.SendAddonMessage( addonName, syncString, "GUILD" )) then
         cblPrint("Error - report of guild ignored players!")
      end
   elseif (words[1] == "test") then 
      cblPrint("This is for developer use only :)")
   else
      cblPrint("Error - type /cbl help for help!")
   end
end 
